import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang3.SerializationUtils;
import org.springframework.beans.BeanUtils;

import com.fasterxml.jackson.databind.ObjectMapper;

public class Test {
    
    static class Object1 implements Cloneable, Serializable {
        private Object2 object2;
        public Object1() {}
        public Object1(Object2 object2) {
            setObject2(object2);
        }
        public Object2 getObject2() {
            return object2;
        }
        public void setObject2(Object2 object2) {
            this.object2 = object2;
        }
        public Object clone() throws CloneNotSupportedException {
            return super.clone();
        }
    }
    
    static class Object2 implements Cloneable, Serializable {
        private Map<String, String> map;
        public Object2() {}
        public Object2(Map<String, String> map) {
            setMap(map);
        }
        public Map<String, String> getMap() {
            return map;
        }
        public void setMap(Map<String, String> map) {
            this.map = map;
        }
        public Object clone() throws CloneNotSupportedException {
            return super.clone();
        }
    }
    
    public static void main(String[] args) throws Exception {
        ObjectMapper objectMapper = new ObjectMapper();
        
        Map<String, String> map = new HashMap<>();
        map.put("test", "t1");
        
        Object1 object1 = new Object1(new Object2(map));
        Object1 newObject1 = null;
        
        // implements Cloneable, clone() method 필요
        newObject1 = (Object1) object1.clone();
        map.put("test", "t2");
        System.out.println("--------------------------------------");
        System.out.println("# object1.clone();");
        System.out.println(objectMapper.writeValueAsString(object1));
        System.out.println(objectMapper.writeValueAsString(newObject1));
        
        // implements Serializable 필요
        newObject1 = SerializationUtils.clone(object1);
        map.put("test", "t3");
        System.out.println("--------------------------------------");
        System.out.println("# SerializationUtils.clone(object1);");
        System.out.println(objectMapper.writeValueAsString(object1));
        System.out.println(objectMapper.writeValueAsString(newObject1));
        
        // spring framework 필요
        BeanUtils.copyProperties(object1 , newObject1);
        map.put("test", "t4");
        System.out.println("--------------------------------------");
        System.out.println("# BeanUtils.copyProperties(object1 , newObject1);");
        System.out.println(objectMapper.writeValueAsString(object1));
        System.out.println(objectMapper.writeValueAsString(newObject1));
        
        // argument가 없는 기본 생성자 필요
        newObject1    = objectMapper.readValue(objectMapper.writeValueAsString(object1), Object1.class);
        map.put("test", "t5");
        System.out.println("--------------------------------------");
        System.out.println("# objectMapper.readValue(objectMapper.writeValueAsString(object1), Object1.class);");
        System.out.println(objectMapper.writeValueAsString(object1));
        System.out.println(objectMapper.writeValueAsString(newObject1));
        
        /*  결과값
            --------------------------------------
            # object1.clone();
            {"object2":{"map":{"test":"t2"}}}
            {"object2":{"map":{"test":"t2"}}}
            --------------------------------------
            # SerializationUtils.clone(object1);
            {"object2":{"map":{"test":"t3"}}}
            {"object2":{"map":{"test":"t2"}}}
            --------------------------------------
            # BeanUtils.copyProperties(object1 , newObject1);
            {"object2":{"map":{"test":"t4"}}}
            {"object2":{"map":{"test":"t4"}}}
            --------------------------------------
            # objectMapper.readValue(objectMapper.writeValueAsString(object1), Object1.class);
            {"object2":{"map":{"test":"t5"}}}
            {"object2":{"map":{"test":"t4"}}}
         */
    }
}
